<?php

namespace App\Controller\OgBoot\PxeBootFile;

use App\Controller\OgBoot\AbstractOgBootController;
use App\Dto\Input\PxeTemplateSyncClientInput;
use App\Entity\Client;
use App\Entity\PxeTemplate;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Attribute\AsController;
use Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;
use Symfony\Contracts\HttpClient\HttpClientInterface;

#[AsController]
class PostAction extends AbstractOgBootController
{
    /**
     * @throws TransportExceptionInterface
     * @throws ServerExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws ClientExceptionInterface
     */
    public function __invoke(Client $client, PxeTemplate $pxeTemplate): JsonResponse
    {
        $ogRepoIp = $this->ogBootApiUrl;

        if ($client->getRepository()) {
            $ogRepoIp = $client->getRepository()->getIp();
        } else if ($client->getOrganizationalUnit()->getNetworkSettings()->getRepository()) {
            $ogRepoIp = $client->getOrganizationalUnit()->getNetworkSettings()->getRepository()->getIp();
        }

        $params = [
            'json' => [
                'template_name' => $pxeTemplate->getName(),
                'mac' => strtolower($client->getMac()),
                'lang' => 'es_ES.UTF_8',
                'ip' => $client->getIp(),
                'server_ip' => $this->ogBootApiUrl,
                'router' =>  $client->getOrganizationalUnit()->getNetworkSettings()->getRouter(),
                'netmask' => $client->getOrganizationalUnit()->getNetworkSettings() ? $client->getOrganizationalUnit()->getNetworkSettings()->getNetmask() : '255.255.255.0',
                'computer_name' => $client->getName(),
                'netiface' => $client->getNetiface(),
                'group' => $client->getOrganizationalUnit()->getName(),
                'ogrepo' => $ogRepoIp,
                'ogcore' => $this->ogCoreIP,
                'oglive' => $this->ogBootApiUrl,
                'oglog' => $this->ogLogIp,
                'ogshare' => $client->getOrganizationalUnit()->getNetworkSettings()?->getOgShare()
                    ? $client->getOrganizationalUnit()->getNetworkSettings()?->getOgShare(): $this->ogBootApiUrl,
                'oglivedir' => $client->getOgLive()->getFilename(),
                'ogprof' => 'false',
                'hardprofile' => $client->getHardwareProfile() ? $client->getHardwareProfile()->getDescription() : 'default',
                'ogntp' => $client->getOrganizationalUnit()->getNetworkSettings()?->getNtp(),
                'ogdns' => $client->getOrganizationalUnit()->getNetworkSettings()?->getDns(),
                'ogProxy' => $client->getOrganizationalUnit()->getNetworkSettings()?->getProxy(),
                'resolution' => '788'
            ]
        ];

        $content = $this->createRequest('POST', 'http://'.$this->ogBootApiUrl.'/ogboot/v1/pxes', $params);

        $client->setPxeSync(true);
        $this->entityManager->persist($client);
        $this->entityManager->flush();

        return new JsonResponse(data: $content, status: Response::HTTP_OK);
    }
}